Package: ambiorix
Title: Web Framework Inspired by 'Express.js'
Version: 2.2.0
Authors@R: 
  c(
    person(given = "John",
           family = "Coene",
           role = c("aut", "cre"),
           email = "jcoenep@gmail.com",
           comment = c(ORCID = "0000-0002-6637-4107")),
    person(family = "Opifex", role = c("fnd")),
    person(given = "Kennedy", family = "Mwavu", role = "ctb", comment = c(ORCID = "0009-0006-3157-7234"))
  )
Description: A web framework inspired by 'express.js' to build
  any web service from multi-page websites to 'RESTful' 
  application programming interfaces.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: fs, log, cli, glue, httpuv, methods, assertthat, webutils (>=
        1.2.0), yyjsonr (>= 0.1.20)
Suggests: mime, readr, readxl, ggplot2, promises, jsonlite, websocket,
        htmltools, commonmark, htmlwidgets, testthat (>= 3.0.0)
URL: https://github.com/ambiorix-web/ambiorix, https://ambiorix.dev
BugReports: https://github.com/ambiorix-web/ambiorix/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-03-13 10:51:06 UTC; mwavu
Author: John Coene [aut, cre] (<https://orcid.org/0000-0002-6637-4107>),
  Opifex [fnd],
  Kennedy Mwavu [ctb] (<https://orcid.org/0009-0006-3157-7234>)
Maintainer: John Coene <jcoenep@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-15 16:40:02 UTC
Built: R 4.3.3; ; 2025-03-18 02:10:56 UTC; unix
