Package: ambiorix
Title: Web Framework Inspired by 'Express.js'
Version: 3.0.0
Authors@R: 
  c(
    person(given = "John",
           family = "Coene",
           role = "aut",
           email = "jcoenep@gmail.com",
           comment = c(ORCID = "0000-0002-6637-4107")),
    person(family = "Opifex", role = c("fnd")),
    person(given = "Kennedy", family = "Mwavu", role = "cre", email = "mwavukennedy@gmail.com", comment = c(ORCID = "0009-0006-3157-7234")),
    person(given = "Julio", family = "Collazos", role = "ctb", comment = c(ORCID = "0009-0006-5503-0997"))
  )
Description: A web framework inspired by 'express.js' to build
  any web service from multi-page websites to 'RESTful' 
  application programming interfaces.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: log, cli, glue, httpuv, methods, assertthat, webutils (>=
        1.2.0), yyjsonr (>= 0.1.20), mime (>= 0.13), promises (>=
        1.5.0)
Suggests: readr, readxl, ggplot2, jsonlite, websocket, htmltools,
        commonmark, htmlwidgets, testthat (>= 3.0.0)
URL: https://github.com/ambiorix-web/ambiorix, https://ambiorix.dev
BugReports: https://github.com/ambiorix-web/ambiorix/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-25 03:44:23 UTC; mwavu
Author: John Coene [aut] (ORCID: <https://orcid.org/0000-0002-6637-4107>),
  Opifex [fnd],
  Kennedy Mwavu [cre] (ORCID: <https://orcid.org/0009-0006-3157-7234>),
  Julio Collazos [ctb] (ORCID: <https://orcid.org/0009-0006-5503-0997>)
Maintainer: Kennedy Mwavu <mwavukennedy@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-25 04:00:03 UTC
Built: R 4.6.0; ; 2026-04-26 05:31:27 UTC; unix
