# Generated by roxygen2: do not edit by hand

S3method(convert_body,default)
S3method(convert_body,factor)
S3method(convert_body,shiny.tag)
S3method(convert_body,shiny.tag.list)
S3method(print,ambiorixResponse)
S3method(print,cookie)
S3method(print,cookieParser)
S3method(print,cookiePreprocessor)
S3method(print,forward)
S3method(print,jobj)
S3method(print,pathToPattern)
S3method(print,renderer)
S3method(print,responsePreHook)
S3method(print,robj)
export(Ambiorix)
export(Request)
export(Response)
export(Router)
export(Websocket)
export(as_cookie_parser)
export(as_cookie_preprocessor)
export(as_path_to_pattern)
export(as_renderer)
export(content_csv)
export(content_html)
export(content_json)
export(content_plain)
export(content_protobuf)
export(content_tsv)
export(copy_websocket_client)
export(default_cookie_parser)
export(forward)
export(import)
export(jobj)
export(mockRequest)
export(new_log)
export(parse_form_urlencoded)
export(parse_json)
export(parse_multipart)
export(pre_hook)
export(response)
export(response_404)
export(response_500)
export(robj)
export(serialise)
export(set_log_error)
export(set_log_info)
export(set_log_success)
export(stop_all)
export(token_create)
export(use_html_template)
importFrom(assertthat,assert_that)
importFrom(methods,formalArgs)
importFrom(utils,browseURL)
