% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{use_html_template}
\alias{use_html_template}
\title{HTML Template}
\usage{
use_html_template()
}
\value{
A renderer function.
}
\description{
Use \code{\link[htmltools:htmlTemplate]{htmltools::htmlTemplate()}} as renderer.
Passed to \code{use} method.
}
\examples{
use_html_template()
}
