% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stop.R
\name{stop_all}
\alias{stop_all}
\title{Stop}
\usage{
stop_all()
}
\value{
\code{NULL} (invisibly)
}
\description{
Stop all servers.
}
\examples{
if (interactive()) {
  stop_all()
}
}
