% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/content.R
\name{content}
\alias{content}
\alias{content_html}
\alias{content_plain}
\alias{content_json}
\alias{content_csv}
\alias{content_tsv}
\alias{content_protobuf}
\title{Content Headers}
\usage{
content_html()

content_plain()

content_json()

content_csv()

content_tsv()

content_protobuf()
}
\value{
Length 1 character vector.
}
\description{
Convenient functions for more readable content type headers.
}
\examples{
list(
 "Content-Type",
 content_json()
)

if(FALSE)
 req$header(
  "Content-Type",
  content_json()
 )

}
