% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{as_renderer}
\alias{as_renderer}
\title{Create a Renderer}
\usage{
as_renderer(fn)
}
\arguments{
\item{fn}{A function that accepts two arguments,
the full path to the \code{file} to render, and the
\code{data} to render.}
}
\value{
A renderer function.
}
\description{
Create a custom renderer.
}
\examples{
if (interactive()) {
  fn <- function(path, data) {
    # ...
  }

  as_renderer(fn)
}
}
