% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{import}
\alias{import}
\title{Import Files}
\usage{
import(...)
}
\arguments{
\item{...}{Directory from which to import \code{.R} or \code{.r} files.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Import all R-files in a directory.
}
\examples{
if (interactive()) {
  import("views")
}

}
