% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/request.R
\name{mockRequest}
\alias{mockRequest}
\title{Mock Request}
\usage{
mockRequest(cookie = "", query = "", path = "/")
}
\arguments{
\item{cookie}{Cookie string.}

\item{query}{Query string.}

\item{path}{Path string.}
}
\value{
A \code{Request} object.
}
\description{
Mock a request, used for tests.
}
\examples{
mockRequest()

}
