% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log.R
\name{new_log}
\alias{new_log}
\title{Logger}
\usage{
new_log(prefix = ">", write = FALSE, file = "ambiorix.log", sep = "")
}
\arguments{
\item{prefix}{String to prefix all log messages.}

\item{write}{Whether to write the log to the \code{file}.}

\item{file}{Name of the file to dump the logs to,
only used if \code{write} is \code{TRUE}.}

\item{sep}{Separator between \code{prefix} and other
flags and messages.}
}
\value{
An R& of class \code{log::Logger}.
}
\description{
Returns a new logger using the \code{log} package.
}
\examples{
log <- new_log()
log$log("Hello world")

}
