% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/token.R
\name{token_create}
\alias{token_create}
\title{Token}
\usage{
token_create(n = 16L)
}
\arguments{
\item{n}{Number of bytes.}
}
\value{
Length 1 character vector.
}
\description{
Create a token
}
\examples{
token_create()
token_create(n = 32L)
}
