% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{robj}
\alias{robj}
\title{R Object}
\usage{
robj(obj)
}
\arguments{
\item{obj}{R object to treat.}
}
\value{
Object of class "robj".
}
\description{
Treats a data element rendered in a response (\code{res$render}) as
a data object and ultimately uses \code{\link[=dput]{dput()}}.
}
\details{
For instance in a template, \verb{x <- [\% var \%]} will not work with
\code{res$render(data=list(var = "hello"))} because this will be replace
like \code{x <- hello} (missing quote): breaking the template. Using \code{robj} one would
obtain \code{x <- "hello"}.
}
\examples{
robj(1:10)
}
