% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forward.R
\name{forward}
\alias{forward}
\title{Forward Method}
\usage{
forward()
}
\value{
An object of class \code{forward}.
}
\description{
Makes it such that the web server skips this method and uses the next one in line instead.
}
\examples{
app <- Ambiorix$new()

app$get("/next", function(req, res){
 forward()
})

app$get("/next", function(req, res){
 res$send("Hello")
})

if(interactive())
 app$start()

}
