% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{jobj}
\alias{jobj}
\title{JSON Object}
\usage{
jobj(obj)
}
\arguments{
\item{obj}{Object to serialise.}
}
\value{
Object of class "jobj".
}
\description{
Serialises an object to JSON in \code{res$render}.
}
\examples{
if (interactive()) {
  l <- list(a = "hello", b = 2L, c = 3)
  jobj(l)
}
}
